@@ -1,7 +1,7 @@
 #!/bin/bash
 # Copyright 1999-2007 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: ebuild.sh 10330 2008-05-13 23:09:03Z zmedico $
+# $Id: ebuild.sh 12137 2008-12-02 22:39:22Z zmedico $
 
 PORTAGE_BIN_PATH="${PORTAGE_BIN_PATH:-/usr/lib/portage/bin}"
 PORTAGE_PYM_PATH="${PORTAGE_PYM_PATH:-/usr/lib/portage/pym}"
@@ -166,7 +166,7 @@ has_version() {
 	fi
 	# return shell-true/shell-false if exists.
 	# Takes single depend-type atoms.
-	PYTHONPATH="${PORTAGE_PYM_PATH}:${PYTHONPATH}" \
+	PYTHONPATH=${PORTAGE_PYM_PATH}${PYTHONPATH:+:}${PYTHONPATH} \
 	"${PORTAGE_BIN_PATH}"/portageq has_version "${ROOT}" "$1"
 	local retval=$?
 	case "${retval}" in
@@ -186,7 +186,7 @@ portageq() {
 	if [ "${EBUILD_PHASE}" == "depend" ]; then
 		die "portageq calls are not allowed in the global scope"
 	fi
-	PYTHONPATH="${PORTAGE_PYM_PATH}:${PYTHONPATH}" \
+	PYTHONPATH=${PORTAGE_PYM_PATH}${PYTHONPATH:+:}${PYTHONPATH} \
 	"${PORTAGE_BIN_PATH}/portageq" "$@"
 }
 
@@ -202,8 +202,20 @@ best_version() {
 	fi
 	# returns the best/most-current match.
 	# Takes single depend-type atoms.
-	PYTHONPATH="${PORTAGE_PYM_PATH}:${PYTHONPATH}" \
+	PYTHONPATH=${PORTAGE_PYM_PATH}${PYTHONPATH:+:}${PYTHONPATH} \
 	"${PORTAGE_BIN_PATH}/portageq" 'best_version' "${ROOT}" "$1"
+	local retval=$?
+	case "${retval}" in
+		0)
+			return 0
+			;;
+		1)
+			return 1
+			;;
+		*)
+			die "unexpected portageq exit code: ${retval}"
+			;;
+	esac
 }
 
 use_with() {
@@ -260,6 +272,18 @@ register_die_hook() {
 	export EBUILD_DEATH_HOOKS="${EBUILD_DEATH_HOOKS} $*"
 }
 
+# Ensure that $PWD is sane whenever possible, to protect against
+# exploitation of insecure search path for python -c in ebuilds.
+# See bug #239560.
+if ! hasq "$EBUILD_PHASE" clean cleanrm depend help ; then
+	cd "$PORTAGE_BUILDDIR" || \
+		die "PORTAGE_BUILDDIR does not exist: '$PORTAGE_BUILDDIR'"
+else
+	# Don't try to create this when it's parent
+	# directory doesn't necessarily exist.
+	unset EBUILD_EXIT_STATUS_FILE
+fi
+
 #if no perms are specified, dirs/files will have decent defaults
 #(not secretive, but not stupid)
 umask 022
@@ -421,6 +445,10 @@ econf() {
 	local x
 	local LOCAL_EXTRA_ECONF="${EXTRA_ECONF}"
 
+	! hasq "$EAPI" 0 1 && [[ $EBUILD_PHASE = compile && \
+		$(type -t src_configure) = function ]] && \
+		eqawarn "econf called in src_compile instead of src_configure"
+
 	if [ -z "${ECONF_SOURCE}" ]; then
 		ECONF_SOURCE="."
 	fi
@@ -471,32 +499,7 @@ econf() {
 			LOCAL_EXTRA_ECONF="--libdir=${CONF_LIBDIR_RESULT} ${LOCAL_EXTRA_ECONF}"
 		fi
 
-		local TMP_CONFCACHE_DIR CONFCACHE_ARG
-		if hasq confcache $FEATURES && ! hasq confcache $RESTRICT; then
-			CONFCACHE="$(type -P confcache)"
-			if [ -z "${CONFCACHE}" ]; then
-				ewarn "disabling confcache, binary cannot be found"
-			else
-				CONFCACHE="${CONFCACHE/ /\ }"
-				TMP_CONFCACHE_DIR="${CONFCACHE:+${CONFCACHE_DIR:-${PORTAGE_TMPDIR}/confcache}}"
-				TMP_CONFCACHE_DIR="${TMP_CONFCACHE_DIR/ /\ }"
-				CONFCACHE_ARG="--confcache-dir"
-				local s
-				if [ -n "$CCACHE_DIR" ]; then
-					s="$CCACHE_DIR"
-				fi
-				if [ -n "$DISTCC_DIR" ]; then
-					s="${s:+${s}:}$DISTCC_DIR"
-				fi
-				if [ -n "$s" ]; then
-					CONFCACHE_ARG="--confcache-ignore $s $CONFCACHE_ARG"
-				fi
-			fi
-		else
-			CONFCACHE=
-		fi
-
-		vecho ${CONFCACHE} ${CONFCACHE_ARG} ${TMP_CONFCACHE_DIR} "${ECONF_SOURCE}/configure" \
+		vecho "${ECONF_SOURCE}/configure" \
 			--prefix=/usr \
 			--host=${CHOST} \
 			--mandir=/usr/share/man \
@@ -507,7 +510,7 @@ econf() {
 			"$@" \
 			${LOCAL_EXTRA_ECONF}
 
-		if ! ${CONFCACHE} ${CONFCACHE_ARG} ${TMP_CONFCACHE_DIR} "${ECONF_SOURCE}/configure" \
+		if ! "${ECONF_SOURCE}/configure" \
 			--prefix=/usr \
 			--host=${CHOST} \
 			--mandir=/usr/share/man \
@@ -571,7 +574,7 @@ einstall() {
 	fi
 }
 
-pkg_nofetch() {
+_eapi0_pkg_nofetch() {
 	[ -z "${SRC_URI}" ] && return
 
 	echo "!!! The following are listed in SRC_URI for ${PN}:"
@@ -581,22 +584,18 @@ pkg_nofetch() {
 	done
 }
 
-src_unpack() {
+_eapi0_src_unpack() {
 	[[ -n ${A} ]] && unpack ${A}
 }
 
-src_compile() {
-	if [ "${EAPI:-0}" == 0 ] ; then
-		[ -x ./configure ] && econf
-	elif [ -x "${ECONF_SOURCE:-.}/configure" ] ; then
+_eapi0_src_compile() {
+	if [ -x ./configure ] ; then
 		econf
 	fi
-	if [ -f Makefile ] || [ -f GNUmakefile ] || [ -f makefile ]; then
-		emake || die "emake failed"
-	fi
+	_eapi2_src_compile
 }
 
-src_test() {
+_eapi0_src_test() {
 	if emake -j1 check -n &> /dev/null; then
 		vecho ">>> Test phase [check]: ${CATEGORY}/${PF}"
 		if ! emake -j1 check; then
@@ -614,13 +613,21 @@ src_test() {
 	fi
 }
 
-# Used to generate the /lib/cpp and /usr/bin/cc wrappers
-gen_wrapper() {
-	cat > "$1" <<-EOF
-	#!/bin/sh
-	exec $2 "\$@"
-	EOF
-	chmod 0755 "$1"
+_eapi1_src_compile() {
+	_eapi2_src_configure
+	_eapi2_src_compile
+}
+
+_eapi2_src_configure() {
+	if [[ -x ${ECONF_SOURCE:-.}/configure ]] ; then
+		econf
+	fi
+}
+
+_eapi2_src_compile() {
+	if [ -f Makefile ] || [ -f GNUmakefile ] || [ -f makefile ]; then
+		emake || die "emake failed"
+	fi
 }
 
 ebuild_phase() {
@@ -710,12 +717,9 @@ dyn_clean() {
 	fi
 
 	if ! hasq keepwork $FEATURES; then
-		rm -rf "${PORTAGE_BUILDDIR}/.exit_status"
-		rm -rf "${PORTAGE_BUILDDIR}/.logid"
-		rm -rf "${PORTAGE_BUILDDIR}/.unpacked"
-		rm -rf "${PORTAGE_BUILDDIR}/.compiled"
-		rm -rf "${PORTAGE_BUILDDIR}/.tested"
-		rm -rf "${PORTAGE_BUILDDIR}/.packaged"
+		rm -f "$PORTAGE_BUILDDIR"/.{exit_status,logid,unpacked,prepared} \
+			"$PORTAGE_BUILDDIR"/.{configured,compiled,tested,packaged}
+
 		rm -rf "${PORTAGE_BUILDDIR}/build-info"
 		rm -rf "${WORKDIR}"
 	fi
@@ -728,9 +732,10 @@ dyn_clean() {
 	# result in it wiping the users distfiles directory (bad).
 	rm -rf "${PORTAGE_BUILDDIR}/distdir"
 
-	if [ -z "$(find "${PORTAGE_BUILDDIR}" -mindepth 1 -maxdepth 1)" ]; then
-		rmdir "${PORTAGE_BUILDDIR}"
-	fi
+	# Some kernels, such as Solaris, return EINVAL when an attempt
+	# is made to remove the current working directory.
+	cd "$PORTAGE_BUILDDIR"/../..
+	rmdir "$PORTAGE_BUILDDIR" "${PORTAGE_BUILDDIR%/*}" 2>/dev/null
 
 	true
 }
@@ -819,6 +824,18 @@ abort_handler() {
 	trap SIGINT SIGQUIT
 }
 
+abort_prepare() {
+	abort_handler src_prepare $1
+	rm -f "$PORTAGE_BUILDDIR/.prepared"
+	exit 1
+}
+
+abort_configure() {
+	abort_handler src_configure $1
+	rm -f "$PORTAGE_BUILDDIR/.configured"
+	exit 1
+}
+
 abort_compile() {
 	abort_handler "src_compile" $1
 	rm -f "${PORTAGE_BUILDDIR}/.compiled"
@@ -837,78 +854,74 @@ abort_install() {
 	exit 1
 }
 
-dyn_compile() {
-	trap "abort_compile" SIGINT SIGQUIT
+dyn_prepare() {
 
-	[ "$(type -t pre_src_compile)" == "function" ] && qa_call pre_src_compile
-
-	[ "${CFLAGS-unset}"      != "unset" ] && export CFLAGS
-	[ "${CXXFLAGS-unset}"    != "unset" ] && export CXXFLAGS
-	[ "${LIBCFLAGS-unset}"   != "unset" ] && export LIBCFLAGS
-	[ "${LIBCXXFLAGS-unset}" != "unset" ] && export LIBCXXFLAGS
-	[ "${LDFLAGS-unset}"     != "unset" ] && export LDFLAGS
-	[ "${ASFLAGS-unset}"     != "unset" ] && export ASFLAGS
-
-	[ "${CCACHE_DIR-unset}"  != "unset" ] && export CCACHE_DIR
-	[ "${CCACHE_SIZE-unset}" != "unset" ] && export CCACHE_SIZE
-
-	[ "${DISTCC_DIR-unset}"  == "unset" ] && export DISTCC_DIR="${PORTAGE_TMPDIR}/.distcc"
-	[ ! -z "${DISTCC_DIR}" ] && addwrite "${DISTCC_DIR}"
-
-	LIBDIR_VAR="LIBDIR_${ABI}"
-	if [ -z "${PKG_CONFIG_PATH}" -a -n "${ABI}" -a -n "${!LIBDIR_VAR}" ]; then
-		export PKG_CONFIG_PATH="/usr/${!LIBDIR_VAR}/pkgconfig"
-	fi
-	unset LIBDIR_VAR
-
-	if hasq noauto $FEATURES && [ ! -f ${PORTAGE_BUILDDIR}/.unpacked ]; then
-		echo
-		echo "!!! We apparently haven't unpacked... This is probably not what you"
-		echo "!!! want to be doing... You are using FEATURES=noauto so I'll assume"
-		echo "!!! that you know what you are doing... You have 5 seconds to abort..."
-		echo
-
-		local x
-		for x in 1 2 3 4 5 6 7 8; do
-			echo -ne "\a"
-			LC_ALL=C sleep 0.25
-		done
-
-		sleep 3
+	if [[ $PORTAGE_BUILDDIR/.prepared -nt $WORKDIR ]] ; then
+		vecho ">>> It appears that '$PF' is already prepared; skipping."
+		vecho ">>> Remove '$PORTAGE_BUILDDIR/.prepared' to force prepare."
+		return 0
 	fi
 
-	local srcdir=${PORTAGE_BUILDDIR}
-	cd "${PORTAGE_BUILDDIR}"
-	if [ ! -e "build-info" ]; then
-		mkdir build-info
-	fi
-	cp "${EBUILD}" "build-info/${PF}.ebuild"
-
-	if [[ ${PORTAGE_BUILDDIR}/.compiled -nt ${WORKDIR} ]] ; then
-		vecho ">>> It appears that '${PF}' is already compiled; skipping."
-		vecho ">>> Remove '${PORTAGE_BUILDDIR}/.compiled' to force compilation."
-		trap SIGINT SIGQUIT
-		[ "$(type -t post_src_compile)" == "function" ] && qa_call post_src_compile
-		return
-	fi
-	if [ -d "${S}" ]; then
-		srcdir=${S}
+	local srcdir
+	if [[ -d $S ]] ; then
+		srcdir=$S
 	else
-		srcdir=${WORKDIR}
+		srcdir=$WORKDIR
 	fi
-	cd "${srcdir}"
-	#our custom version of libtool uses $S and $D to fix
-	#invalid paths in .la files
-	export S D
-	#some packages use an alternative to $S to build in, cause
-	#our libtool to create problematic .la files
-	export PWORKDIR="$WORKDIR"
+	cd "$srcdir"
+
+	trap abort_prepare SIGINT SIGQUIT
+
+	ebuild_phase pre_src_prepare
+	ebuild_phase src_prepare
+	touch "$PORTAGE_BUILDDIR"/.prepared
+	ebuild_phase post_src_prepare
+
+	trap SIGINT SIGQUIT
+}
+
+dyn_configure() {
+
+	if [[ $PORTAGE_BUILDDIR/.configured -nt $WORKDIR ]] ; then
+		vecho ">>> It appears that '$PF' is already configured; skipping."
+		vecho ">>> Remove '$PORTAGE_BUILDDIR/.configured' to force configuration."
+		return 0
+	fi
+
+	trap abort_configure SIGINT SIGQUIT
+
+	[[ $(type -t pre_src_configure) = function ]] && \
+		qa_call pre_src_configure
+
+	vecho ">>> Configuring source in $srcdir ..."
+	ebuild_phase src_configure
+	touch "$PORTAGE_BUILDDIR"/.configured
+	vecho ">>> Source configured."
+
+	[[ $(type -t post_src_configure) = function ]] && \
+		qa_call post_src_configure
+
+	trap SIGINT SIGQUIT
+}
+
+dyn_compile() {
+
+	if [[ $PORTAGE_BUILDDIR/.compiled -nt $WORKDIR ]] ; then
+		vecho ">>> It appears that '${PF}' is already compiled; skipping."
+		vecho ">>> Remove '$PORTAGE_BUILDDIR/.compiled' to force compilation."
+		return 0
+	fi
+
+	trap abort_compile SIGINT SIGQUIT
+
+	[[ $(type -t pre_src_compile) = function ]] && \
+		qa_call pre_src_compile
+
 	vecho ">>> Compiling source in ${srcdir} ..."
 	ebuild_phase src_compile
+	touch "$PORTAGE_BUILDDIR"/.compiled
 	vecho ">>> Source compiled."
-	#|| abort_compile "fail"
-	cd "${PORTAGE_BUILDDIR}"
-	touch .compiled
+
 	[ "$(type -t post_src_compile)" == "function" ] && qa_call post_src_compile
 
 	trap SIGINT SIGQUIT
@@ -998,7 +1011,7 @@ dyn_install() {
 	for f in ASFLAGS CATEGORY CBUILD CC CFLAGS CHOST CTARGET CXX \
 		CXXFLAGS DEPEND EXTRA_ECONF EXTRA_EINSTALL EXTRA_MAKE \
 		FEATURES INHERITED IUSE LDFLAGS LIBCFLAGS LIBCXXFLAGS \
-		LICENSE PDEPEND PF PKGUSE PROVIDE RDEPEND RESTRICT SLOT \
+		LICENSE PDEPEND PF PKGUSE PROPERTIES PROVIDE RDEPEND RESTRICT SLOT \
 		KEYWORDS HOMEPAGE SRC_URI DESCRIPTION; do
 		[ -n "${!f}" ] && echo $(echo "${!f}" | \
 			tr '\n,\r,\t' ' , , ' | sed s/'  \+'/' '/g) > ${f}
@@ -1010,15 +1023,9 @@ dyn_install() {
 	# local variables can leak into the saved environment.
 	unset f
 
-	(
-		# To avoid environment.bz2 bloat, cleanse variables that are
-		# are no longer needed after src_install(). Don't cleanse from
-		# the global environment though, in case the user wants to repeat
-		# this phase (like with FEATURES=noauto and the ebuild command).
-		unset S _E_DOCDESTTREE_ _E_EXEDESTTREE_
-		save_ebuild_env | filter_readonly_variables \
-			--filter-sandbox --allow-extra-vars > environment
-	)
+	save_ebuild_env --exclude-init-phases | filter_readonly_variables \
+		--filter-sandbox --allow-extra-vars > environment
+
 	bzip2 -f9 environment
 
 	cp "${EBUILD}" "${PF}.ebuild"
@@ -1041,7 +1048,7 @@ dyn_preinst() {
 dyn_help() {
 	echo
 	echo "Portage"
-	echo "Copyright 1999-2006 Gentoo Foundation"
+	echo "Copyright 1999-2008 Gentoo Foundation"
 	echo
 	echo "How to use the ebuild command:"
 	echo
@@ -1055,9 +1062,11 @@ dyn_help() {
 	echo "  fetch       : download source archive(s) and patches"
 	echo "  digest      : create a manifest file for the package"
 	echo "  manifest    : create a manifest file for the package"
-	echo "  unpack      : unpack/patch sources (auto-fetch if needed)"
-	echo "  compile     : compile sources (auto-fetch/unpack if needed)"
-	echo "  test        : test package (auto-fetch/unpack/compile if needed)"
+	echo "  unpack      : unpack sources (auto-dependencies if needed)"
+	echo "  prepare     : prepare sources (auto-dependencies if needed)"
+	echo "  configure   : configure sources (auto-fetch/unpack if needed)"
+	echo "  compile     : compile sources (auto-fetch/unpack/configure if needed)"
+	echo "  test        : test package (auto-fetch/unpack/configure/compile if needed)"
 	echo "  preinst     : execute pre-install instructions"
 	echo "  postinst    : execute post-install instructions"
 	echo "  install     : install the package to the temporary install directory"
@@ -1173,7 +1182,7 @@ inherit() {
 			# false alarms due to INHERITED in /var/db/pkg being outdated
 			# in comparison the the eclasses from the portage tree.
 			if ! hasq $ECLASS $INHERITED; then
-				eqawarn "QA Notice: ECLASS '$ECLASS' inherited illegally in $CATEGORY/$PF"
+				eqawarn "QA Notice: ECLASS '$ECLASS' inherited illegally in $CATEGORY/$PF $EBUILD_PHASE"
 			fi
 		fi
 
@@ -1260,7 +1269,7 @@ EXPORT_FUNCTIONS() {
 	fi
 	while [ "$1" ]; do
 		debug-print "EXPORT_FUNCTIONS: ${1} -> ${ECLASS}_${1}"
-		eval "$1() { ${ECLASS}_$1 "\$@" ; }" > /dev/null
+		eval "$1() { ${ECLASS}_$1 \"\$@\" ; }" > /dev/null
 		shift
 	done
 }
@@ -1347,19 +1356,178 @@ remove_path_entry() {
 	PATH="${stripped_path}"
 }
 
-source_all_bashrcs() {
-	local OCC="${CC}" OCXX="${CXX}"
-	# source the existing profile.bashrc's.
-	save_IFS
-	IFS=$'\n'
-	local x
-	for x in ${PROFILE_PATHS}; do
-		# Must unset it so that it doesn't mess up assumptions in the RCs.
-		unset IFS
-		[ -f "${x}/profile.bashrc" ] && qa_source "${x}/profile.bashrc"
+# @FUNCTION: _ebuild_arg_to_phase
+# @DESCRIPTION:
+# Translate a known ebuild(1) argument into the precise
+# name of it's corresponding ebuild phase.
+_ebuild_arg_to_phase() {
+	[ $# -ne 2 ] && die "expected exactly 2 args, got $#: $*"
+	local eapi=$1
+	local arg=$2
+	local phase_func=""
+
+	case "$arg" in
+		setup)
+			phase_func=pkg_setup
+			;;
+		nofetch)
+			phase_func=pkg_nofetch
+			;;
+		unpack)
+			phase_func=src_unpack
+			;;
+		prepare)
+			! hasq $eapi 0 1 2_pre1 2_pre2 && \
+				phase_func=src_prepare
+			;;
+		configure)
+			! hasq $eapi 0 1 2_pre1 && \
+				phase_func=src_configure
+			;;
+		compile)
+			phase_func=src_compile
+			;;
+		test)
+			phase_func=src_test
+			;;
+		install)
+			phase_func=src_install
+			;;
+		preinst)
+			phase_func=pkg_preinst
+			;;
+		postinst)
+			phase_func=pkg_postinst
+			;;
+		prerm)
+			phase_func=pkg_prerm
+			;;
+		postrm)
+			phase_func=pkg_postrm
+			;;
+	esac
+
+	[[ -z $phase_func ]] && return 1
+	echo "$phase_func"
+	return 0
+}
+
+_ebuild_phase_funcs() {
+	[ $# -ne 2 ] && die "expected exactly 2 args, got $#: $*"
+	local eapi=$1
+	local phase_func=$2
+	local default_phases="pkg_nofetch src_unpack src_prepare src_configure
+		src_compile src_install src_test"
+	local x y default_func=""
+
+	for x in pkg_nofetch src_unpack src_test ; do
+		[[ $(type -t $x) = function ]] || \
+			eval "$x() { _eapi0_$x \"\$@\" ; }"
 	done
-	restore_IFS
-	
+
+	case $eapi in
+
+		0|1|2_pre1)
+
+			if [[ $(type -t src_compile) != function ]] ; then
+				case $eapi in
+					0)
+						src_compile() { _eapi0_src_compile "$@" ; }
+						;;
+					*)
+						src_compile() { _eapi1_src_compile "$@" ; }
+						;;
+				esac
+			fi
+
+			for x in $default_phases ; do
+				eval "default_$x() {
+					die \"default_$x() is not supported with EAPI='$eapi' during phase $phase_func\"
+				}"
+			done
+
+			eval "default() {
+				die \"default() is not supported with EAPI='$eapi' during phase $phase_func\"
+			}"
+
+			;;
+
+		*)
+
+			[[ $(type -t src_configure) = function ]] || \
+				src_configure() { _eapi2_src_configure "$@" ; }
+
+			[[ $(type -t src_compile) = function ]] || \
+				src_compile() { _eapi2_src_compile "$@" ; }
+
+			if hasq $phase_func $default_phases ; then
+
+				_eapi2_pkg_nofetch   () { _eapi0_pkg_nofetch          "$@" ; }
+				_eapi2_src_unpack    () { _eapi0_src_unpack           "$@" ; }
+				_eapi2_src_prepare   () { true                             ; }
+				_eapi2_src_test      () { _eapi0_src_test             "$@" ; }
+				_eapi2_src_install   () { die "$FUNCNAME is not supported" ; }
+
+				for x in $default_phases ; do
+					eval "default_$x() { _eapi2_$x \"\$@\" ; }"
+				done
+
+				eval "default() { _eapi2_$phase_func \"\$@\" ; }"
+
+			else
+
+				for x in $default_phases ; do
+					eval "default_$x() {
+						die \"default_$x() is not supported in phase $default_func\"
+					}"
+				done
+
+				eval "default() {
+					die \"default() is not supported with EAPI='$eapi' during phase $phase_func\"
+				}"
+
+			fi
+
+			;;
+	esac
+}
+
+PORTAGE_BASHRCS_SOURCED=0
+
+# @FUNCTION: source_all_bashrcs
+# @DESCRIPTION:
+# Source a relevant bashrc files and perform other miscellaneous
+# environment initialization when appropriate.
+#
+# If EAPI is set then define functions provided by the current EAPI:
+#
+#  * default_* aliases for the current EAPI phase functions
+#  * A "default" function which is an alias for the default phase
+#    function for the current phase.
+#
+source_all_bashrcs() {
+	[[ $PORTAGE_BASHRCS_SOURCED = 1 ]] && return 0
+	PORTAGE_BASHRCS_SOURCED=1
+	local x
+
+	if [[ -n $EBUILD_PHASE && -n $EAPI ]] ; then
+		local phase_func=$(_ebuild_arg_to_phase "$EAPI" $EBUILD_PHASE)
+		[[ -n $phase_func ]] && _ebuild_phase_funcs "$EAPI" $phase_func
+	fi
+
+	local OCC="${CC}" OCXX="${CXX}"
+
+	if [[ $EBUILD_PHASE != depend ]] ; then
+		# source the existing profile.bashrcs.
+		save_IFS
+		IFS=$'\n'
+		local path_array=($PROFILE_PATHS)
+		restore_IFS
+		for x in "${path_array[@]}" ; do
+			[ -f "$x/profile.bashrc" ] && qa_source "$x/profile.bashrc"
+		done
+	fi
+
 	# We assume if people are changing shopts in their bashrc they do so at their
 	# own peril.  This is the ONLY non-portage bit of code that can change shopts
 	# without a QA violation.
@@ -1438,7 +1606,7 @@ filter_readonly_variables() {
 		SANDBOX_DEBUG_LOG SANDBOX_DISABLED SANDBOX_LIB
 		SANDBOX_LOG SANDBOX_ON"
 	filtered_vars="${readonly_bash_vars} ${READONLY_PORTAGE_VARS}
-		BASH_.* PATH"
+		BASH_.* PATH POSIXLY_CORRECT"
 	if hasq --filter-sandbox $* ; then
 		filtered_vars="${filtered_vars} SANDBOX_.*"
 	else
@@ -1636,7 +1804,7 @@ if [[ -n ${QA_INTERCEPTORS} ]] ; then
 	unset BIN_PATH BIN BODY FUNC_SRC
 fi
 
-if ! hasq ${EBUILD_PHASE} clean depend && \
+if ! hasq "$EBUILD_PHASE" clean cleanrm depend && \
 	[ -f "${T}"/environment ] ; then
 	# The environment may have been extracted from environment.bz2 or
 	# may have come from another version of ebuild.sh or something.
@@ -1679,10 +1847,9 @@ if ! hasq ${EBUILD_PHASE} clean depend && \
 			;;
 	esac
 
-	source_all_bashrcs
 fi
 
-if ! hasq ${EBUILD_PHASE} clean && \
+if ! hasq "$EBUILD_PHASE" clean cleanrm && \
 	(
 		hasq ${EBUILD_PHASE} depend || \
 		[ ! -f "${T}"/environment ] || \
@@ -1723,8 +1890,9 @@ if ! hasq ${EBUILD_PHASE} clean && \
 	set +f
 fi
 
-# enable bashrc support for the clean phase
-[[ ${EBUILD_PHASE} == clean ]] && source_all_bashrcs
+# Set default EAPI if necessary, so that most
+# code can simply assume that it's defined.
+[[ -n $EAPI ]] || EAPI=0
 
 # unset USE_EXPAND variables that contain only the special "*" token
 for x in ${USE_EXPAND} ; do
@@ -1752,7 +1920,12 @@ if [ "${EBUILD_PHASE}" != "depend" ] ; then
 	declare -r ${READONLY_EBUILD_METADATA} ${READONLY_PORTAGE_VARS}
 fi
 
-if [ -n "${EBUILD_SH_ARGS}" ] ; then
+ebuild_main() {
+	source_all_bashrcs
+	local f x
+	local export_vars="ASFLAGS CCACHE_DIR CCACHE_SIZE
+		CFLAGS CXXFLAGS LDFLAGS LIBCFLAGS LIBCXXFLAGS"
+
 	case ${EBUILD_SH_ARGS} in
 	nofetch)
 		ebuild_phase_with_hooks pkg_nofetch
@@ -1774,20 +1947,76 @@ if [ -n "${EBUILD_SH_ARGS}" ] ; then
 		if [[ $EBUILD_PHASE == postinst ]] && [[ -n $PORTAGE_UPDATE_ENV ]]; then
 			# Update environment.bz2 in case installation phases
 			# need to pass some variables to uninstallation phases.
-			(
-				unset S _E_DOCDESTTREE_ _E_EXEDESTTREE_
-				save_ebuild_env | filter_readonly_variables \
-					--filter-sandbox --allow-extra-vars | \
-					bzip2 -c -f9 > "$PORTAGE_UPDATE_ENV"
-			)
+			save_ebuild_env --exclude-init-phases | \
+				filter_readonly_variables --filter-sandbox --allow-extra-vars \
+				| bzip2 -c -f9 > "$PORTAGE_UPDATE_ENV"
 		fi
 		;;
-	unpack|compile|test|clean|install)
-		if [ "${SANDBOX_DISABLED="0"}" == "0" ]; then
+	unpack|prepare|configure|compile|test|clean|install)
+		if [[ ${SANDBOX_DISABLED:-0} = 0 ]] ; then
 			export SANDBOX_ON="1"
 		else
 			export SANDBOX_ON="0"
 		fi
+
+		case "$EBUILD_SH_ARGS" in
+		configure|compile)
+
+			for x in $export_vars ; do
+				[[ ${!x-unset} != unset ]] && export $x
+			done
+
+			hasq distcc $FEATURES && [[ -n $DISTCC_DIR ]] && \
+				[[ ${SANDBOX_WRITE/$DISTCC_DIR} = $SANDBOX_WRITE ]] && \
+				addwrite "$DISTCC_DIR"
+
+			x=LIBDIR_$ABI
+			[ -z "$PKG_CONFIG_PATH" -a -n "$ABI" -a -n "${!x}" ] && \
+				export PKG_CONFIG_PATH=/usr/${!x}/pkgconfig
+
+			if hasq noauto $FEATURES && \
+				[[ ! -f $PORTAGE_BUILDDIR/.unpacked ]] ; then
+				echo
+				echo "!!! We apparently haven't unpacked..." \
+					"This is probably not what you"
+				echo "!!! want to be doing... You are using" \
+					"FEATURES=noauto so I'll assume"
+				echo "!!! that you know what you are doing..." \
+					"You have 5 seconds to abort..."
+				echo
+
+				local x
+				for x in 1 2 3 4 5 6 7 8; do
+					echo -ne "\a"
+					LC_ALL=C sleep 0.25
+				done
+
+				sleep 3
+			fi
+
+			cd "$PORTAGE_BUILDDIR"
+			if [ ! -d build-info ] ; then
+				mkdir build-info
+				cp "$EBUILD" "build-info/$PF.ebuild"
+			fi
+
+			local srcdir
+			if [[ -d $S ]] ; then
+				srcdir=$S
+			else
+				srcdir=$WORKDIR
+			fi
+			cd "$srcdir"
+			#our custom version of libtool uses $S and $D to fix
+			#invalid paths in .la files
+			export S D
+			#some packages use an alternative to $S to build in, cause
+			#our libtool to create problematic .la files
+			export PWORKDIR=$WORKDIR
+
+			;;
+		esac
+
 		if [ "${PORTAGE_DEBUG}" != "1" ] || [ "${-/x/}" != "$-" ]; then
 			dyn_${EBUILD_SH_ARGS}
 		else
@@ -1824,8 +2053,8 @@ if [ -n "${EBUILD_SH_ARGS}" ] ; then
 
 		auxdbkeys="DEPEND RDEPEND SLOT SRC_URI RESTRICT HOMEPAGE LICENSE
 			DESCRIPTION KEYWORDS INHERITED IUSE CDEPEND PDEPEND PROVIDE EAPI
-			UNUSED_01 UNUSED_02 UNUSED_03 UNUSED_04 UNUSED_05 UNUSED_06
-			UNUSED_07"
+			PROPERTIES UNUSED_06 UNUSED_05 UNUSED_04
+			UNUSED_03 UNUSED_02 UNUSED_01"
 
 		#the extra $(echo) commands remove newlines
 		unset CDEPEND
@@ -1851,9 +2080,13 @@ if [ -n "${EBUILD_SH_ARGS}" ] ; then
 		exit 1
 		;;
 	esac
-	[ -n "${EBUILD_EXIT_STATUS_FILE}" ] && \
-		touch "${EBUILD_EXIT_STATUS_FILE}" &>/dev/null
-fi
+	if [ -n "$EBUILD_EXIT_STATUS_FILE" ] ; then
+		> "$EBUILD_EXIT_STATUS_FILE" || \
+			die "failed to create '$EBUILD_EXIT_STATUS_FILE'"
+	fi
+}
+
+[[ -n $EBUILD_SH_ARGS ]] && ebuild_main
 
 # Save the env only for relevant phases.
 if [ -n "${EBUILD_SH_ARGS}" ] && \
