@@ -1,6 +1,6 @@
 # Copyright 1999-2006 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: isolated-functions.sh 10197 2008-05-05 06:03:43Z zmedico $
+# $Id: isolated-functions.sh 12135 2008-12-02 22:20:34Z zmedico $
 
 # We need this next line for "die" and "assert". It expands
 # It _must_ preceed all the calls to die and assert.
@@ -134,8 +134,7 @@ die() {
 		done
 	fi
 
-	[ -n "${EBUILD_EXIT_STATUS_FILE}" ] && \
-		touch "${EBUILD_EXIT_STATUS_FILE}" &>/dev/null
+	[ -n "$EBUILD_EXIT_STATUS_FILE" ] && > "$EBUILD_EXIT_STATUS_FILE"
 
 	# subshell die support
 	kill -s SIGTERM ${EBUILD_MASTER_PID}
@@ -170,19 +169,30 @@ elog_base() {
 			return 1
 			;;
 	esac
-	echo -ne "${messagetype} $*\n\0" >> "${T}/logging/${EBUILD_PHASE:-other}"
+	echo -e "$@" | while read ; do
+		echo "$messagetype $REPLY" >> \
+			"${T}/logging/${EBUILD_PHASE:-other}"
+	done
 	return 0
 }
 
 eqawarn() {
 	elog_base QA "$*"
-	vecho -e " ${WARN}*${NORMAL} $*" >&2
+	[[ ${RC_ENDCOL} != "yes" && ${LAST_E_CMD} == "ebegin" ]] && echo
+	echo -e "$@" | while read ; do
+		vecho -e " $WARN*$NORMAL $REPLY" >&2
+	done
+	LAST_E_CMD="eqawarn"
 	return 0
 }
 
 elog() {
 	elog_base LOG "$*"
-	echo -e " ${GOOD}*${NORMAL} $*"
+	[[ ${RC_ENDCOL} != "yes" && ${LAST_E_CMD} == "ebegin" ]] && echo
+	echo -e "$@" | while read ; do
+		echo -e " $GOOD*$NORMAL $REPLY"
+	done
+	LAST_E_CMD="elog"
 	return 0
 }
 
@@ -207,7 +217,9 @@ esyslog() {
 einfo() {
 	elog_base INFO "$*"
 	[[ ${RC_ENDCOL} != "yes" && ${LAST_E_CMD} == "ebegin" ]] && echo
-	echo -e " ${GOOD}*${NORMAL} $*"
+	echo -e "$@" | while read ; do
+		echo -e " $GOOD*$NORMAL $REPLY"
+	done
 	LAST_E_CMD="einfo"
 	return 0
 }
@@ -223,7 +235,9 @@ einfon() {
 ewarn() {
 	elog_base WARN "$*"
 	[[ ${RC_ENDCOL} != "yes" && ${LAST_E_CMD} == "ebegin" ]] && echo
-	echo -e " ${WARN}*${NORMAL} ${RC_INDENTATION}$*" >&2
+	echo -e "$@" | while read ; do
+		echo -e " $WARN*$NORMAL $RC_INDENTATION$REPLY" >&2
+	done
 	LAST_E_CMD="ewarn"
 	return 0
 }
@@ -231,7 +245,9 @@ ewarn() {
 eerror() {
 	elog_base ERROR "$*"
 	[[ ${RC_ENDCOL} != "yes" && ${LAST_E_CMD} == "ebegin" ]] && echo
-	echo -e " ${BAD}*${NORMAL} ${RC_INDENTATION}$*" >&2
+	echo -e "$@" | while read ; do
+		echo -e " $BAD*$NORMAL $RC_INDENTATION$REPLY" >&2
+	done
 	LAST_E_CMD="eerror"
 	return 0
 }
@@ -281,6 +297,7 @@ eend() {
 
 	_eend ${retval} eerror "$*"
 
+	LAST_E_CMD="eend"
 	return ${retval}
 }
 
@@ -353,7 +370,7 @@ set_colors() {
 	(( COLS > 0 )) || (( COLS = 80 ))
 	COLS=$((${COLS} - 8))	# width of [ ok ] == 7
 	# Adjust COLS so that eend works properly on a standard BSD console.
-	[ "${TERM}" = "cons25" ] && COLS=$((${COLS} - 1))
+	[[ $TERM = cons25 || $TERM = dumb ]] && ((COLS--))
 
 	# Now, ${ENDCOL} will move us to the end of the
 	# column;  irregardless of character width
@@ -425,9 +442,24 @@ hasq() {
 # @FUNCTION: save_ebuild_env
 # @DESCRIPTION:
 # echo the current environment to stdout, filtering out redundant info.
+#
+# --exclude-init-phases causes pkg_nofetch and src_* phase functions to
+# be excluded from the output. These function are not needed for installation
+# or removal of the packages, and can therefore be safely excluded.
+#
 save_ebuild_env() {
 	(
 
+		if hasq --exclude-init-phases $* ; then
+			unset S _E_DOCDESTTREE_ _E_EXEDESTTREE_
+			unset -f pkg_nofetch src_unpack src_prepare src_configure \
+			src_compile src_test src_install
+			if [[ -n $PYTHONPATH ]] ; then
+				export PYTHONPATH=${PYTHONPATH/${PORTAGE_PYM_PATH}:}
+				[[ -z $PYTHONPATH ]] && unset PYTHONPATH
+			fi
+		fi
+
 		# misc variables set by bash
 		unset BASH HOSTTYPE IFS MACHTYPE OLDPWD \
 			OPTERR OPTIND OSTYPE PS4 PWD SHELL SHLVL
@@ -446,21 +478,32 @@ save_ebuild_env() {
 		# There's no need to bloat environment.bz2 with internally defined
 		# functions and variables, so filter them out if possible.
 
+		for x in pkg_setup pkg_nofetch src_unpack src_prepare src_configure \
+			src_compile src_test src_install pkg_preinst pkg_postinst \
+			pkg_prerm pkg_postrm ; do
+			unset -f default_$x _eapi{0,1,2}_$x
+		done
+		unset x
+
 		unset -f dump_trace die diefunc quiet_mode vecho elog_base eqawarn elog \
 			esyslog einfo einfon ewarn eerror ebegin _eend eend KV_major \
 			KV_minor KV_micro KV_to_int get_KV unset_colors set_colors has \
 			hasv hasq qa_source qa_call addread addwrite adddeny addpredict \
 			lchown lchgrp esyslog use usev useq has_version portageq \
 			best_version use_with use_enable register_die_hook check_KV \
-			keepdir unpack strip_duplicate_slashes econf einstall gen_wrapper \
+			keepdir unpack strip_duplicate_slashes econf einstall \
 			dyn_setup dyn_unpack dyn_clean into insinto exeinto docinto \
-			insopts diropts exeopts libopts abort_handler abort_compile \
-			abort_test abort_install dyn_compile dyn_test dyn_install \
+			insopts diropts exeopts libopts \
+			abort_handler abort_prepare abort_configure abort_compile \
+			abort_test abort_install dyn_prepare dyn_configure \
+			dyn_compile dyn_test dyn_install \
 			dyn_preinst dyn_help debug-print debug-print-function \
 			debug-print-section inherit EXPORT_FUNCTIONS newdepend newrdepend \
 			newpdepend do_newdepend remove_path_entry \
 			save_ebuild_env filter_readonly_variables preprocess_ebuild_env \
-			source_all_bashrcs ebuild_phase ebuild_phase_with_hooks \
+			source_all_bashrcs ebuild_main \
+			ebuild_phase ebuild_phase_with_hooks \
+			_ebuild_arg_to_phase _ebuild_phase_funcs default \
 			${QA_INTERCEPTORS}
 
 		# portage config variables and variables set directly by portage
@@ -471,13 +514,15 @@ save_ebuild_env() {
 			GOOD HILITE HOME IMAGE \
 			LAST_E_CMD LAST_E_LEN LD_PRELOAD MISC_FUNCTIONS_ARGS MOPREFIX \
 			NORMAL PKGDIR PKGUSE PKG_LOGDIR PKG_TMPDIR \
-			PORTAGE_ACTUAL_DISTDIR PORTAGE_ARCHLIST PORTAGE_BASHRC \
+			PORTAGE_ACTUAL_DISTDIR PORTAGE_ARCHLIST \
+			PORTAGE_BASHRC PORTAGE_BASHRCS_SOURCED \
 			PORTAGE_BINPKG_TAR_OPTS PORTAGE_BINPKG_TMPFILE PORTAGE_BUILDDIR \
 			PORTAGE_COLORMAP PORTAGE_CONFIGROOT PORTAGE_DEBUG \
 			PORTAGE_DEPCACHEDIR PORTAGE_GID PORTAGE_INST_GID \
 			PORTAGE_INST_UID PORTAGE_LOG_FILE PORTAGE_MASTER_PID \
+			PORTAGE_QUIET \
 			PORTAGE_REPO_NAME PORTAGE_RESTRICT PORTAGE_UPDATE_ENV \
-			PORTAGE_WORKDIR_MODE PORTDIR \
+			PORTAGE_VERBOSE PORTAGE_WORKDIR_MODE PORTDIR \
 			PORTDIR_OVERLAY ${!PORTAGE_SANDBOX_*} PREROOTPATH \
 			PROFILE_PATHS PWORKDIR QA_INTERCEPTORS \
 			RC_DEFAULT_INDENT RC_DOT_PATTERN RC_ENDCOL \
