@@ -43,7 +43,10 @@ install_symlink_html_docs() {
 
 install_qa_check() {
 	cd "${D}" || die "cd failed"
+
 	prepall
+	ecompressdir --dequeue
+	ecompress --dequeue
 
 	# Now we look for all world writable files.
 	for i in $(find "${D}/" -type f -perm -2); do
@@ -57,6 +60,7 @@ install_qa_check() {
 
 	if type -P scanelf > /dev/null && ! hasq binchecks ${RESTRICT}; then
 		local qa_var insecure_rpath=0 tmp_quiet=${PORTAGE_QUIET}
+		local f x
 
 		# display warnings when using stricter because we die afterwards
 		if has stricter ${FEATURES} ; then
@@ -102,8 +106,7 @@ install_qa_check() {
 			eqawarn " risk.  On some architectures, the code may not even function"
 			eqawarn " properly, if at all."
 			eqawarn " For more information, see http://hardened.gentoo.org/pic-fix-guide.xml"
-			eqawarn " Please include this file in your report:"
-			eqawarn " ${T}/scanelf-textrel.log"
+			eqawarn " Please include the following list of files in your report:"
 			eqawarn "${f}"
 			vecho -ne '\a\n'
 			die_msg="${die_msg} textrels,"
@@ -147,16 +150,82 @@ install_qa_check() {
 			eqawarn " on some architectures/operating systems.  A bug should be filed"
 			eqawarn " at http://bugs.gentoo.org/ to make sure the file is fixed."
 			eqawarn " For more information, see http://hardened.gentoo.org/gnu-stack.xml"
-			eqawarn " Please include this file in your report:"
-			eqawarn " ${T}/scanelf-execstack.log"
+			eqawarn " Please include the following list of files in your report:"
 			eqawarn "${f}"
 			vecho -ne '\a\n'
 			die_msg="${die_msg} execstacks"
 			sleep 1
 		fi
 
-		# Save NEEDED information
-		scanelf -qyRF '%p %n' "${D}" | sed -e 's:^:/:' > "${PORTAGE_BUILDDIR}"/build-info/NEEDED
+		# Check for files built without respecting LDFLAGS
+		if [[ "${LDFLAGS}" == *--hash-style=gnu* ]] && [[ "${PN}" != *-bin ]] ; then
+			f=$(scanelf -qyRF '%k %p' -k .hash "${D}" | sed -e "s:\.hash ::")
+			if [[ -n ${f} ]] ; then
+				echo "${f}" > "${T}"/scanelf-ignored-LDFLAGS.log
+				if [ "${QA_STRICT_DT_HASH-unset}" == unset ] ; then
+					if [[ ${#QA_DT_HASH[@]} -gt 1 ]] ; then
+						for x in "${QA_DT_HASH[@]}" ; do
+							sed -e "s#^${x#/}\$##" -i "${T}"/scanelf-ignored-LDFLAGS.log
+						done
+					else
+						local shopts=$-
+						set -o noglob
+						for x in ${QA_DT_HASH} ; do
+							sed -e "s#^${x#/}\$##" -i "${T}"/scanelf-ignored-LDFLAGS.log
+						done
+						set +o noglob
+						set -${shopts}
+					fi
+				fi
+				# Filter anything under /usr/lib/debug/ in order to avoid
+				# duplicate warnings for splitdebug files.
+				sed -e "s#^usr/lib/debug/.*##" -e "/^\$/d" -e "s#^#/#" \
+					-i "${T}"/scanelf-ignored-LDFLAGS.log
+				f=$(<"${T}"/scanelf-ignored-LDFLAGS.log)
+				if [[ -n ${f} ]] ; then
+					vecho -ne '\a\n'
+					eqawarn "${BAD}QA Notice: Files built without respecting LDFLAGS have been detected${NORMAL}"
+					eqawarn " Please include the following list of files in your report:"
+					eqawarn "${f}"
+					vecho -ne '\a\n'
+					sleep 1
+				else
+					rm -f "${T}"/scanelf-ignored-LDFLAGS.log
+				fi
+			fi
+		fi
+
+		# Save NEEDED information after removing self-contained providers
+		scanelf -qyRF '%a;%p;%S;%r;%n' "${D}" | { while IFS= read l; do
+			arch=${l%%;*}; l=${l#*;}
+			obj="/${l%%;*}"; l=${l#*;}
+			soname=${l%%;*}; l=${l#*;}
+			rpath=${l%%;*}; l=${l#*;}; [ "${rpath}" = "  -  " ] && rpath=""
+			needed=${l%%;*}; l=${l#*;}
+			if [ -z "${rpath}" -o -n "${rpath//*ORIGIN*}" ]; then
+				# object doesn't contain $ORIGIN in its runpath attribute
+				echo "${obj} ${needed}"	>> "${PORTAGE_BUILDDIR}"/build-info/NEEDED
+				echo "${arch:3};${obj};${soname};${rpath};${needed}" >> "${PORTAGE_BUILDDIR}"/build-info/NEEDED.ELF.2
+			else
+				dir=${obj%/*}
+				# replace $ORIGIN with the dirname of the current object for the lookup
+				opath=$(echo :${rpath}: | sed -e "s#.*:\(.*\)\$ORIGIN\(.*\):.*#\1${dir}\2#")
+				sneeded=$(echo ${needed} | tr , ' ')
+				rneeded=""
+				for lib in ${sneeded}; do
+					found=0
+					for path in ${opath//:/ }; do
+						[ -e "${D}/${path}/${lib}" ] && found=1 && break
+					done
+					[ "${found}" -eq 0 ] && rneeded="${rneeded},${lib}"
+				done
+				rneeded=${rneeded:1}
+				if [ -n "${rneeded}" ]; then
+					echo "${obj} ${rneeded}" >> "${PORTAGE_BUILDDIR}"/build-info/NEEDED
+					echo "${arch:3};${obj};${soname};${rpath};${rneeded}" >> "${PORTAGE_BUILDDIR}"/build-info/NEEDED.ELF.2
+				fi
+			fi
+		done }
 
 		if [[ ${insecure_rpath} -eq 1 ]] ; then
 			die "Aborting due to serious QA concerns with RUNPATH/RPATH"
@@ -269,7 +338,7 @@ install_qa_check() {
 	fi
 
 	# Verify that the libtool files don't contain bogus $D entries.
-	abort="no"
+	local abort=no gentoo_bug=no
 	for a in "${D}"usr/lib*/*.la ; do
 		s=${a##*/}
 		if grep -qs "${D}" "${a}" ; then
@@ -307,14 +376,38 @@ install_qa_check() {
 		done
 		f=$(cat "${PORTAGE_LOG_FILE}" | check-implicit-pointer-usage.py)
 		if [[ -n ${f} ]] ; then
-			vecho -ne '\a\n'
-			eqawarn "QA Notice: Package has poor programming practices which may compile"
-			eqawarn "           but will almost certainly crash on 64bit architectures."
-			eqawarn "${f}"
-			vecho -ne '\a\n'
-			abort="yes"
+
+			# In the future this will be a forced "die". In preparation,
+			# increase the log level from "qa" to "eerror" so that people
+			# are aware this is a problem that must be fixed asap.
+
+			# just warn on 32bit hosts but bail on 64bit hosts
+			case ${CHOST} in
+				alpha*|ia64*|powerpc64*|mips64*|sparc64*|x86_64*) gentoo_bug=yes ;;
+			esac
+
+			abort=yes
+
+			if [[ $gentoo_bug = yes ]] ; then
+				eerror
+				eerror "QA Notice: Package has poor programming practices which may compile"
+				eerror "           but will almost certainly crash on 64bit architectures."
+				eerror
+				eerror "${f}"
+				eerror
+				eerror " Please file a bug about this at http://bugs.gentoo.org/"
+				eerror " with the maintaining herd of the package."
+				eerror
+			else
+				vecho -ne '\a\n'
+				eqawarn "QA Notice: Package has poor programming practices which may compile"
+				eqawarn "           but will almost certainly crash on 64bit architectures."
+				eqawarn "${f}"
+				vecho -ne '\a\n'
+			fi
+
 		fi
-		if [[ ${abort} == "yes" ]] ; then
+		if [[ $abort = yes ]] && [[ $gentoo_bug != yes ]] ; then
 			echo "Please do not file a Gentoo bug and instead" \
 			"report the above QA issues directly to the upstream" \
 			"developers of this software." | fmt -w 70 | \
@@ -535,30 +628,29 @@ dyn_package() {
 	# in there in case any tools were built with -pg in CFLAGS.
 	cd "${T}"
 	install_mask "${PORTAGE_BUILDDIR}/image" "${PKG_INSTALL_MASK}"
-	local pkg_dest="${PKGDIR}/All/${PF}.tbz2"
-	local pkg_tmp="${PKGDIR}/All/${PF}.tbz2.$$"
 	local tar_options=""
-	[ "${PORTAGE_QUIET}" == "1" ] ||  tar_options="${tar_options} -v"
+	[[ $PORTAGE_VERBOSE = 1 ]] && tar_options+=" -v"
 	# Sandbox is disabled in case the user wants to use a symlink
 	# for $PKGDIR and/or $PKGDIR/All.
 	export SANDBOX_ON="0"
+	[ -z "${PORTAGE_BINPKG_TMPFILE}" ] && \
+		PORTAGE_BINPKG_TMPFILE="${PKGDIR}/${CATEGORY}/${PF}.tbz2"
+	mkdir -p "${PORTAGE_BINPKG_TMPFILE%/*}" || die "mkdir failed"
 	tar $tar_options -cf - $PORTAGE_BINPKG_TAR_OPTS -C "${D}" . | \
-		bzip2 -f > "${pkg_tmp}" || \
-		die "Failed to create tarball"
-	export PYTHONPATH=${PORTAGE_PYM_PATH:-/usr/lib/portage/pym}
-	python -c "import xpak; t=xpak.tbz2('${pkg_tmp}'); t.recompose('${PORTAGE_BUILDDIR}/build-info')"
+		bzip2 -f > "$PORTAGE_BINPKG_TMPFILE"
+	assert
+	PYTHONPATH=${PORTAGE_PYM_PATH}${PYTHONPATH:+:}${PYTHONPATH} \
+	python -c "from portage import xpak; t=xpak.tbz2('${PORTAGE_BINPKG_TMPFILE}'); t.recompose('${PORTAGE_BUILDDIR}/build-info')"
 	if [ $? -ne 0 ]; then
-		rm -f "${pkg_tmp}"
+		rm -f "${PORTAGE_BINPKG_TMPFILE}"
 		die "Failed to append metadata to the tbz2 file"
 	fi
-	mv -f "${pkg_tmp}" "${pkg_dest}" || die "Failed to move tbz2 to ${pkg_dest}"
-	ln -sf "../All/${PF}.tbz2" "${PKGDIR}/${CATEGORY}/${PF}.tbz2" || die "Failed to create symlink in ${PKGDIR}/${CATEGORY}"
 	local md5_hash=""
 	if type md5sum &>/dev/null ; then
-		md5_hash=$(md5sum "${pkg_dest}")
+		md5_hash=$(md5sum "${PORTAGE_BINPKG_TMPFILE}")
 		md5_hash=${md5_hash%% *}
 	elif type md5 &>/dev/null ; then
-		md5_hash=$(md5 "${pkg_dest}")
+		md5_hash=$(md5 "${PORTAGE_BINPKG_TMPFILE}")
 		md5_hash=${md5_hash##* }
 	fi
 	[ -n "${md5_hash}" ] && \
@@ -569,7 +661,9 @@ dyn_package() {
 }
 
 dyn_spec() {
-	tar czf "/usr/src/rpm/SOURCES/${PF}.tar.gz" \
+	local sources_dir=/usr/src/rpm/SOURCES
+	mkdir -p "${sources_dir}"
+	tar czf "${sources_dir}/${PF}.tar.gz" \
 		"${EBUILD}" "${FILESDIR}" || \
 		die "Failed to create base rpm tarball."
 
@@ -603,16 +697,20 @@ __END1__
 }
 
 dyn_rpm() {
+	cd "${T}" || die "cd failed"
+	local machine_name=$(uname -m)
+	local dest_dir=/usr/src/rpm/RPMS/${machine_name}
 	addwrite /usr/src/rpm
 	addwrite "${RPMDIR}"
 	dyn_spec
 	rpmbuild -bb --clean --rmsource "${PF}.spec" || die "Failed to integrate rpm spec file"
-	install -D "/usr/src/rpm/RPMS/i386/${PN}-${PV}-${PR}.i386.rpm" \
+	install -D "${dest_dir}/${PN}-${PV}-${PR}.${machine_name}.rpm" \
 		"${RPMDIR}/${CATEGORY}/${PN}-${PV}-${PR}.rpm" || \
 		die "Failed to move rpm"
 }
 
 if [ -n "${MISC_FUNCTIONS_ARGS}" ]; then
+	source_all_bashrcs
 	[ "$PORTAGE_DEBUG" == "1" ] && set -x
 	for x in ${MISC_FUNCTIONS_ARGS}; do
 		${x}
